from ftw.builder import Builder
from ftw.builder import create
from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_INTEGRATION_TESTING
from unittest import TestCase
from zExceptions import BadRequest
from zope.component import queryMultiAdapter
import json


class TestPrincipalRoleTreeUserSearch(TestCase):

    layer = FTW_PERMISSIONMGR_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_raise_badrequest_if_no_search_param_given(self):
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='principal_role_tree_search')

        with self.assertRaises(BadRequest):
            view()

    def test_return_empty_result_if_search_term_is_too_short(self):
        self.portal.REQUEST.set('search_term', 'aa')
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='principal_role_tree_search')

        self.assertEquals('[]', view(), 'Expect an empty result.')

    def test_search_user_by_fullname(self):
        create(Builder('user'))
        self.portal.REQUEST.set('search_term', 'Doe')
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='principal_role_tree_search')

        self.assertEquals(1,
                          len(json.loads(view())),
                          'Expect one entry')

    def test_search_user_by_id(self):
        create(Builder('user'))
        self.portal.REQUEST.set('search_term', 'john.doe')
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='principal_role_tree_search')

        self.assertEquals(1,
                          len(json.loads(view())),
                          'Expect one entry')

    def test_search_result_format(self):
        john = create(Builder('user'))
        john2 = create(Builder('user').named('John', 'Doe2'))

        self.portal.REQUEST.set('search_term', 'Doe')
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='principal_role_tree_search')

        text1 = '{0} ({1})'.format(john.getProperty('fullname'), john.getId())
        text2 = '{0} ({1})'.format(john2.getProperty('fullname'),
                                   john2.getId())
        self.assertEquals(
            [{u'id': john.getId(), u'text': text1},
             {u'id': john2.getId(), u'text': text2}],
            json.loads(view()),
            'Expect to find two users in json parsable format.')

    def test_search_term_with_umlauts(self):
        john = create(Builder('user').named('J\xc3\xb6hn', 'Doe2'))

        self.portal.REQUEST.set('search_term', 'J\xc3\xb6hn')
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='principal_role_tree_search')

        text1 = u'{0} ({1})'.format(
            john.getProperty('fullname').decode('utf-8'),
            john.getId())
        self.assertEquals(
            [{u'id': john.getId(), u'text': text1}],
            json.loads(view()),
            'Expect to find one user with umlauts')



from ftw.builder import Builder
from ftw.builder import create
from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
import transaction
import unittest as unittest


class TestCopyPermissions(unittest.TestCase):

    layer = FTW_PERMISSIONMGR_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def _get_brain(self, obj):
        path = '/'.join(obj.getPhysicalPath())
        return self.portal.portal_catalog({'path': {'query': path, 'depth': 0}})[0]

    @browsing
    def test_copy_permissions_from_user_to_user(self, browser):
        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('user').named('Hugo', 'Boss'))
        folder1.manage_addLocalRoles('hugo.boss', ['Editor'])
        folder2.manage_addLocalRoles('hugo.boss', ['Contributor'])
        folder1.reindexObjectSecurity()
        folder1.reindexObject(idxs=['getId'])
        folder2.reindexObject(idxs=['getId'])

        create(Builder('user').named('Marie', 'Maroon'))

        # search source user
        browser.login().visit(self.portal, view='copy_user_permissions')
        browser.fill({'search_source_user': 'hugo.boss'}).submit()

        # choose source user
        browser.find('Boss Hugo').click()

        # search target user
        browser.fill({'search_target_user': 'marie.maroon'}).submit()

        # choose target user
        browser.find('Maroon Marie').click()

        # test confirm text
        expected_text = ['Are you sure to copy the permissions of user Boss Hugo (hugo.boss) on '
                         'object "Plone site" and all sub objects, to Maroon Marie (marie.maroon)']
        actual_text = browser.css('.text-confirm-copy').normalized_text()
        self.assertEquals(expected_text, actual_text)

        # confirm to copy permissions
        browser.css('button.context').first.click()

        # test redirection
        expected_url = 'http://nohost/plone/@@copy_user_permissions'
        self.assertEquals(expected_url, browser.url)

        # test status message
        self.assertIn('Info Die Berechtigungen wurden kopiert',
                      [node.normalized_text() for node in browser.css('.portalMessage.info')])

        # test copied permissions
        self.assertIn(('marie.maroon', ('Editor',)), folder1.get_local_roles())
        self.assertIn(('marie.maroon', ('Contributor',)), folder2.get_local_roles())

        self.assertEquals(folder1.get_local_roles(), self._get_brain(folder1).get_local_roles)
        self.assertEquals(folder2.get_local_roles(), self._get_brain(folder2).get_local_roles)

    @browsing
    def test_abort_copying_permissions(self, browser):

        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('user').named('Hugo', 'Boss'))
        folder1.manage_addLocalRoles('hugo.boss', ['Editor'])
        folder2.manage_addLocalRoles('hugo.boss', ['Contributor'])
        folder1.reindexObjectSecurity()

        create(Builder('user').named('Marie', 'Maroon'))

        # search source user
        browser.login().visit(self.portal, view='copy_user_permissions')
        browser.fill({'search_source_user': 'hugo.boss'}).submit()

        # choose source user
        browser.find('Boss Hugo').click()

        # search target user
        browser.fill({'search_target_user': 'marie.maroon'}).submit()

        # choose target user
        browser.find('Maroon Marie').click()

        # abort copying permissions
        browser.find('No, abort').click()

        # test redirection
        expected_url = 'http://nohost/plone/@@copy_user_permissions'
        self.assertEquals(expected_url, browser.url)

        # test status message
        self.assertNotIn('Info Die Berechtigungen wurden kopiert',
                         [node.normalized_text() for node in browser.css('.portalMessage.info')])

        # test whether the permission have not been copied
        self.assertNotIn(('marie.maroon', ('Editor',)), folder1.get_local_roles())
        self.assertNotIn(('marie.maroon', ('Contributor',)), folder2.get_local_roles())

    @browsing
    def test_copy_permissions_from_group_to_group(self, browser):
        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('group').titled('Group Oldies'))

        folder1.manage_addLocalRoles('group-oldies', ['Editor'])
        folder2.manage_addLocalRoles('group-oldies', ['Contributor'])
        folder1.reindexObjectSecurity()
        folder1.reindexObject(idxs=['getId'])
        folder2.reindexObject(idxs=['getId'])

        create(Builder('group').titled('Group Newbies'))

        # search source group
        browser.login().visit(self.portal, view='copy_user_permissions')
        browser.fill({'search_source_user': 'group-oldies'}).submit()

        # choose source group
        browser.find('Group Oldies').click()

        # search target group
        browser.fill({'search_target_user': 'group-newbies'}).submit()

        # choose target group
        browser.find('Group Newbies').click()

        # test confirm text
        expected_text = ['Are you sure to copy the permissions of user (group-oldies) on object '
                         '"Plone site" and all sub objects, to (group-newbies)']
        actual_text = browser.css('.text-confirm-copy').normalized_text()
        self.assertEquals(expected_text, actual_text)

        # confirm to copy permissions
        browser.css('button.context').first.click()

        # test redirection
        expected_url = 'http://nohost/plone/@@copy_user_permissions'
        self.assertEquals(expected_url, browser.url)

        # test status message
        self.assertIn('Info Die Berechtigungen wurden kopiert',
                      [node.normalized_text() for node in browser.css('.portalMessage.info')])

        # test copied permissions
        self.assertIn(('group-newbies', ('Editor',)), folder1.get_local_roles())
        self.assertIn(('group-newbies', ('Contributor',)), folder2.get_local_roles())

        self.assertEquals(folder1.get_local_roles(), self._get_brain(folder1).get_local_roles)
        self.assertEquals(folder2.get_local_roles(), self._get_brain(folder2).get_local_roles)

    @browsing
    def test_do_not_overwrite_existing_permissions(self, browser):
        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('user').named('Hugo', 'Boss'))
        create(Builder('user').named('Marie', 'Maroon').with_roles('Reviewer', on=folder1))

        folder1.manage_addLocalRoles('hugo.boss', ['Editor'])
        folder2.manage_addLocalRoles('hugo.boss', ['Contributor'])
        folder1.reindexObjectSecurity()
        folder1.reindexObject(idxs=['getId'])
        folder2.reindexObject(idxs=['getId'])
        transaction.commit()

        # search source user
        browser.login().visit(self.portal, view='copy_user_permissions')
        browser.fill({'search_source_user': 'hugo.boss'}).submit()

        # choose source user
        browser.find('Boss Hugo').click()

        # search target user
        browser.fill({'search_target_user': 'marie.maroon'}).submit()

        # choose target user
        browser.find('Maroon Marie').click()

        # confirm to copy permissions
        browser.css('button.context').first.click()

        # test copied permissions
        self.assertIn(('marie.maroon', ('Reviewer', 'Editor')), folder1.get_local_roles())
        self.assertIn(('marie.maroon', ('Contributor',)), folder2.get_local_roles())



from ftw.builder import Builder
from ftw.builder import create
from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
import transaction
import unittest as unittest


class TestRemovePermissions(unittest.TestCase):

    layer = FTW_PERMISSIONMGR_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    @browsing
    def test_remove_permission_form_user(self, browser):
        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('user').named('Hugo', 'Boss'))
        folder1.manage_addLocalRoles('hugo.boss', ['Editor'])
        folder2.manage_addLocalRoles('hugo.boss', ['Contributor'])
        folder1.reindexObjectSecurity()

        # Approve if user has local roles on folder1 and folder2
        self.assertIn('hugo.boss', dict(folder1.get_local_roles()))
        self.assertIn('hugo.boss', dict(folder2.get_local_roles()))

        # Search user
        browser.login().visit(self.portal, view='remove_user_permissions')
        browser.fill({'search_term': 'hugo.boss'}).submit()

        # Choose user
        browser.find('Boss Hugo').click()

        # test confirm text
        expected_text = ['Are you sure to remove all permissions of Boss Hugo (hugo.boss) '
                         'on object "Plone site" and all sub objects?']
        actual_text = browser.css('.text-confirm-remove').normalized_text()
        self.assertEquals(expected_text, actual_text)

        # Confirm removal of roles
        browser.css('button.context').first.click()

        # After removal, redirect to permission manager overview
        expected_url = 'http://nohost/plone/@@permission_manager'
        self.assertEquals(expected_url, browser.url)

        # Approve if user has no local roles on folder1 and folder2
        self.assertNotIn('hugo.boss', dict(folder1.get_local_roles()))
        self.assertNotIn('hugo.boss', dict(folder2.get_local_roles()))

    @browsing
    def test_remove_permission_form_group(self, browser):
        self.portal = self.layer['portal']

        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('group').titled('Group Oldies'))
        folder1.manage_addLocalRoles('group-oldies', ['Editor'])
        folder2.manage_addLocalRoles('group-oldies', ['Contributor'])
        folder1.reindexObjectSecurity()

        # Search user
        browser.login().visit(self.portal, view='remove_user_permissions')
        browser.fill({'search_term': 'group-oldies'}).submit()

        # Choose user
        browser.find('Group Oldies').click()

        # test confirm text
        expected_text = ['Are you sure to remove all permissions of Group Oldies (group-oldies) '
                         'on object "Plone site" and all sub objects?']
        actual_text = browser.css('.text-confirm-remove').normalized_text()
        self.assertEquals(expected_text, actual_text)

        # Confirm removal of roles
        browser.css('button.context').first.click()

        # After removal, redirect to permission manager overview
        expected_url = 'http://nohost/plone/@@permission_manager'
        self.assertEquals(expected_url, browser.url)

        # Approve if user has no local roles on folder1 and folder2
        self.assertNotIn('group-oldies', dict(folder1.get_local_roles()))
        self.assertNotIn('group-oldies', dict(folder2.get_local_roles()))

    @browsing
    def test_abort_deletion_of_permissions(self, browser):
        folder1 = create(Builder('folder').titled(u'folder1'))
        folder2 = create(Builder('folder').titled(u'folder2').within(folder1))

        create(Builder('group').titled('Group Oldies'))
        folder1.manage_addLocalRoles('group-oldies', ['Editor'])
        folder2.manage_addLocalRoles('group-oldies', ['Contributor'])
        folder1.reindexObjectSecurity()
        transaction.commit()

        # Search user
        browser.login().visit(self.portal, view='remove_user_permissions')
        browser.fill({'search_term': 'group-oldies'}).submit()

        # Choose user
        browser.find('Group Oldies').click()

        # Abort removal of roles
        browser.find('No, abort').click()

        # After abort, redirect to permission manager overview
        expected_url = 'http://nohost/plone/@@remove_user_permissions'
        self.assertEquals(expected_url, browser.url)

        # Approve if user has no local roles on folder1 and folder2
        self.assertIn('group-oldies', dict(folder1.get_local_roles()))
        self.assertIn('group-oldies', dict(folder2.get_local_roles()))



from ftw.permissionmanager import permission_manager_factory as _
from ftw.permissionmanager.treeifier import Treeify
from plone.app.workflow.interfaces import ISharingPageRole
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zExceptions import BadRequest
from zope.component import queryUtility
from zope.i18n import translate
from zope.i18nmessageid import Message
import json
import pkg_resources


try:
    pkg_resources.get_distribution('ftw.lawgiver')
except pkg_resources.DistributionNotFound:
    LAWGIVER_INSTALLED = False
else:
    LAWGIVER_INSTALLED = True
    from ftw.lawgiver.utils import translate_role_for_workflow


def get_friendly_role_name(names, workflow_id, request):
    friendly_names = []

    for name in names:
        if LAWGIVER_INSTALLED:
            title = translate(
                translate_role_for_workflow(workflow_id, name),
                context=request)
            if isinstance(title, Message):
                title = translate(title, context=request)
            friendly_names.append(title)
            continue

        utility = queryUtility(ISharingPageRole, name=name)
        if utility is None:
            friendly_names.append(name)
        else:
            friendly_names.append(translate(utility.title, context=request))
    friendly_names.sort()
    return ', '.join(friendly_names)


class PrincipalRoleTree(BrowserView):

    def __call__(self):
        return self.index()

    def get_preferred_language(self):
        ltool = getToolByName(self.context, 'portal_languages')
        return ltool.getPreferredLanguage().split('-')[0]


class SearchPrincipals(BrowserView):

    def __init__(self, context, request):
        super(SearchPrincipals, self).__init__(context, request)
        self.search_term = None

    def __call__(self):
        self.search_term = self.request.get('search_term', '')

        if not self.search_term:
            raise BadRequest('No search_term found.')

        if len(self.search_term) <= 2:
            return json.dumps([])

        principals = self.search_principals()
        return json.dumps(self._format_result(principals))

    def search_principals(self):
        return self.search_principals_by_fullname() + self.search_principals_by_userid()

    def search_principals_by_fullname(self):
        users = getToolByName(self.context, "acl_users")
        return users.searchUsers(fullname=self.search_term)

    def search_principals_by_userid(self):
        users = getToolByName(self.context, "acl_users")
        return users.searchUsers(id=self.search_term)

    def _format_result(self, principals):
        ids = []
        formatted = []
        for principal in principals:
            userid = principal['userid']
            if principal['userid'] not in ids:
                fullname = principal['title']

                # search for userids returns the userid also as title
                if userid == fullname:
                    mtool = getToolByName(self.context, 'portal_membership')
                    member = mtool.getMemberById(userid)
                    fullname = member and member.getProperty(
                        'fullname', userid) or userid

                # It depends on the mood of plone whether the fullname is utf8
                # encoded or not.
                if not isinstance(fullname, unicode):
                    fullname = fullname.decode('utf-8')
                data = {u'id': userid, 'text': u'{0} ({1})'.format(fullname,
                                                                   userid)}
                formatted.append(data)
                ids.append(userid)

        formatted.sort(key=lambda item: item['text'])
        return formatted


class BuildPrincipalRoleTree(BrowserView):

    item_template = ViewPageTemplateFile('templates/item_template.pt')

    def __init__(self, context, request):
        super(BuildPrincipalRoleTree, self).__init__(context, request)
        self.principalid = None
        self.groupids = []

    def __call__(self):
        mtool = getToolByName(self.context, 'portal_membership')
        gtool = getToolByName(self.context, 'portal_groups')

        principalid = self.request.get('principalid', None)

        if not principalid:
            return ''

        self.principalid = principalid

        member = mtool.getMemberById(self.principalid)
        if member:
            groups = gtool.getGroupsByUserId(self.principalid)
            self.groupids = [group.getId() for group in groups]

        tree = self.build_tree()
        if not tree:
            return translate(_(u'text_no_result', default=u'No results found'),
                             context=self.request)

        else:
            return self.render_tree(tree['children'])

    def render_tree(self, children=[], level=1):
        output = ''
        for node in children:
            output += '<li class="treeItem visualNoMarker">\n'
            output += self.item_template(node=node)
            children = node.get('children', [])
            if len(children):
                output += \
                    '<ul class="level%d">\n%s\n</ul>\n' % (
                        level, self.render_tree(children, level + 1))
            output += '</li>\n'
        return output

    def build_query(self):
        query = {}
        principals = [self.principalid] + self.groupids

        query['principal_with_local_roles'] = principals
        query['sort_on'] = 'getObjPositionInParent'
        query['path'] = {'query': '/'.join(self.context.getPhysicalPath()),
                         'depth': -1}
        return query

    def build_tree(self):
        root_path = '/'.join(self.context.getPhysicalPath())
        brains = self.context.portal_catalog(**self.build_query())

        if not brains:
            return []

        tree = Treeify(brains, root_path, self.node_updater)
        return tree(self.context)

    def node_updater(self, brain, node):
        normalizer = queryUtility(IIDNormalizer)
        node['item'] = brain
        node['user_roles'] = self.get_user_roles(brain)
        node['group_roles'] = self.get_group_roles(brain)
        node['normalized_portaltype'] = normalizer.normalize(brain.portal_type)

    def get_user_roles(self, brain):
        local_roles = brain.get_local_roles
        for principal, roles in local_roles:
            if principal == self.principalid:
                return get_friendly_role_name(roles,
                                              brain.workflow_id,
                                              self.request)
        return ''

    def get_group_roles(self, brain):
        local_roles = brain.get_local_roles
        group_roles = []

        for groupid in self.groupids:
            for principal, roles in local_roles:
                if principal == groupid:
                    gtool = getToolByName(self.context, 'portal_groups')
                    gtitle = gtool.getGroupById(groupid).getProperty(
                        'title',
                        groupid)
                    group_roles.append({'title': gtitle,
                                        'roles': get_friendly_role_name(
                                            roles,
                                            brain.workflow_id,
                                            self.request)})
        return group_roles



from ftw.builder import Builder
from ftw.builder import create
from ftw.permissionmanager.browser import principal_role_tree
from ftw.permissionmanager.indexer import principal_with_local_roles
from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.dexterity.content import Item
from plone.dexterity.fti import DexterityFTI
from plone.dexterity.fti import register
from Products.CMFCore.utils import getToolByName
from unittest import TestCase
from zope.component import queryMultiAdapter
from zope.interface import Interface
import transaction


class TestBuildPrincipalRoleTree(TestCase):

    layer = FTW_PERMISSIONMGR_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_principal_with_local_roles_index(self):
        folder = create(Builder('folder'))
        john = create(Builder('user').with_roles('Reader', on=folder))

        self.assertEquals([john.getId(), 'test_user_1_'],
                          principal_with_local_roles(folder)())

    def test_principal_with_local_roles_index_query(self):
        folder = create(Builder('folder'))
        john = create(Builder('user').with_roles('Reader', on=folder))
        folder.reindexObject()
        catalog = getToolByName(self.portal, 'portal_catalog')
        result = catalog(principal_with_local_roles=[john.getId()])

        self.assertEquals(1,
                          len(result),
                          'Expect one entry')
        self.assertEquals(result[0].getPath(),
                          '/'.join(folder.getPhysicalPath()))

    def test_empty_result_if_no_principal_is_set(self):
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')

        self.assertEquals('', view())

    def _create_structure(self):
        self.folder = create(Builder('folder'))
        self.a = create(Builder('folder').titled(u'a').within(self.folder))
        self.a1 = create(Builder('folder').titled(u'a1').within(self.a))
        self.b = create(Builder('folder').titled(u'b').within(self.folder))
        self.b1 = create(Builder('folder').titled(u'b1').within(self.b))
        self.b11 = create(Builder('folder').titled(u'b11').within(self.b1))

    def test_build_only_branches_with_local_roles(self):
        self._create_structure()
        john = create(Builder('user').with_roles('Reader', on=self.b11))
        self.b11.reindexObject()

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')
        view()
        tree = view.build_tree()['children']

        level1 = tree[0]
        self.assertEquals('/'.join(self.folder.getPhysicalPath()),
                          level1['item'].getPath())

        level2 = level1['children'][0]
        self.assertEquals('/'.join(self.b.getPhysicalPath()),
                          level2['item'].getPath())

        level3 = level2['children'][0]
        self.assertEquals('/'.join(self.b1.getPhysicalPath()),
                          level3['item'].getPath())

        level4 = level3['children'][0]
        self.assertEquals('/'.join(self.b11.getPhysicalPath()),
                          level4['item'].getPath())

    def test_build_branch_with_nested_local_roles(self):
        self._create_structure()
        john = create(Builder('user')
                      .with_roles('Reader', on=self.b11)
                      .with_roles('Contributor', on=self.b))
        self.b11.reindexObject()
        self.b.reindexObject()

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')

        view()
        tree = view.build_tree()['children']

        level1 = tree[0]
        self.assertEquals('/'.join(self.folder.getPhysicalPath()),
                          level1['item'].getPath())

        level2 = level1['children'][0]
        self.assertEquals('/'.join(self.b.getPhysicalPath()),
                          level2['item'].getPath())

        level3 = level2['children'][0]
        self.assertEquals('/'.join(self.b1.getPhysicalPath()),
                          level3['item'].getPath())

        level4 = level3['children'][0]
        self.assertEquals('/'.join(self.b11.getPhysicalPath()),
                          level4['item'].getPath())

    def test_build_only_branches_with_local_roles_from_b(self):
        self._create_structure()
        john = create(Builder('user').with_roles('Reader', on=self.b11))
        self.b11.reindexObject()

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.b, self.b.REQUEST),
                                 name='build_principal_role_tree')
        view()
        tree = view.build_tree()['children']

        level1 = tree[0]
        self.assertEquals('/'.join(self.b1.getPhysicalPath()),
                          level1['item'].getPath())

        level2 = level1['children'][0]
        self.assertEquals('/'.join(self.b11.getPhysicalPath()),
                          level2['item'].getPath())

    def test_build_tree_with_multiple_branches(self):
        self._create_structure()
        john = create(Builder('user')
                      .with_roles('Reader', on=self.b11)
                      .with_roles('Contributor', on=self.a1))
        self.b11.reindexObject()
        self.a1.reindexObject()

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')

        view()
        tree = view.build_tree()['children']

        level1 = tree[0]
        self.assertEquals('/'.join(self.folder.getPhysicalPath()),
                          level1['item'].getPath())

        level2a = level1['children'][0]
        self.assertEquals('/'.join(self.a.getPhysicalPath()),
                          level2a['item'].getPath())

        level2b = level1['children'][1]
        self.assertEquals('/'.join(self.b.getPhysicalPath()),
                          level2b['item'].getPath())

        level3a = level2a['children'][0]
        self.assertEquals('/'.join(self.a1.getPhysicalPath()),
                          level3a['item'].getPath())

    def test_build_tree_with_group_memberships(self):
        self._create_structure()
        john = create(Builder('user'))
        group = create(Builder('group')
                       .titled(u'Group')
                       .with_members(john))

        self.a1.manage_setLocalRoles(group.getId(), ['Contributor'])
        self.a1.reindexObject()

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')

        view()
        tree = view.build_tree()['children']

        level1 = tree[0]
        self.assertEquals('/'.join(self.folder.getPhysicalPath()),
                          level1['item'].getPath())

        level2 = level1['children'][0]
        self.assertEquals('/'.join(self.a.getPhysicalPath()),
                          level2['item'].getPath())

        level3 = level2['children'][0]
        self.assertEquals('/'.join(self.a1.getPhysicalPath()),
                          level3['item'].getPath())

    @browsing
    def test_build_branch_with_local_roles_HTML(self, browser):
        self._create_structure()
        john = create(Builder('user').with_roles('Reader', on=self.b11))
        self.b11.reindexObject()
        transaction.commit()

        data = {'principalid': john.getId()}
        browser.login().visit(self.portal,
                              data=data,
                              view='build_principal_role_tree')

        self.assertEquals('level1',
                          browser.css('ul')[0].attrib['class'],
                          'Expect a level1')
        self.assertEquals('level2',
                          browser.css('ul')[1].attrib['class'],
                          'Expect a level2')
        self.assertEquals('level3',
                          browser.css('ul')[2].attrib['class'],
                          'Expect a level3')

        self.assertTrue(
            browser.css('[href="{0}"]'.format(self.folder.absolute_url())),
            'Expecrt to find the following url {0}'.format(
                self.folder.absolute_url()))
        self.assertTrue(
            browser.css('[href="{0}"]'.format(self.b.absolute_url())),
            'Expecrt to find the following url {0}'.format(
                self.b.absolute_url()))
        self.assertTrue(
            browser.css('[href="{0}"]'.format(self.b1.absolute_url())),
            'Expecrt to find the following url {0}'.format(
                self.b1.absolute_url()))
        self.assertTrue(
            browser.css('[href="{0}"]'.format(self.b11.absolute_url())),
            'Expecrt to find the following url {0}'.format(
                self.b11.absolute_url()))

    @browsing
    def test_build_tree_branch_with_local_roles_HTML_only(self, browser):
        self._create_structure()
        john = create(Builder('user').with_roles('Reader', on=self.b11))
        self.b11.reindexObject()
        transaction.commit()

        data = {'principalid': john.getId()}
        browser.login().visit(self.portal,
                              data=data,
                              view='build_principal_role_tree')

        self.assertFalse(
            browser.css('[href="{0}"]'.format(self.a.absolute_url())),
            'Expecrt NOT to find the following url {0}'.format(
                self.a.absolute_url()))
        self.assertFalse(
            browser.css('[href="{0}"]'.format(self.a1.absolute_url())),
            'Expecrt NOT to find the following url {0}'.format(
                self.a1.absolute_url()))

    @browsing
    def test_item_is_marked_as_not_acquired(self, browser):
        self._create_structure()
        john = create(Builder('user').with_roles('Reader', on=self.b11))
        self.b11.reindexObject()

        setattr(self.b1, '__ac_local_roles_block__', True)
        self.b1.reindexObject()
        transaction.commit()

        data = {'principalid': john.getId()}
        browser.login().visit(self.portal,
                              data=data,
                              view='build_principal_role_tree')

        self.assertEquals(self.b1.Title(),
                          browser.css('div.not-acquired a').first.text)

    @browsing
    def test_empty_result(self, browser):
        self._create_structure()

        data = {'principalid': 'inexistingid'}
        browser.login().visit(self.portal,
                              data=data,
                              view='build_principal_role_tree')

        self.assertEquals(u'No results found', browser.contents)

    def test_get_friendly_role_name_translates_if_utility_is_registered(self):
        self.assertEquals(
            'Can add, Can edit',
            principal_role_tree.get_friendly_role_name(
                ['Contributor', 'Editor'], '', self.portal.REQUEST))

    def test_get_friendly_role_name_does_not_translate(self):
        self.assertEquals(
            'Baar, Foo',
            principal_role_tree.get_friendly_role_name(
                ['Foo', 'Baar'], '', self.portal.REQUEST))

    @browsing
    def test_translated_roles_are_displayed_in_tree(self, browser):
        self._create_structure()
        john = create(Builder('user')
                      .with_roles('Reader', on=self.b11)
                      .with_roles('Contributor', on=self.a1))
        self.b11.reindexObject()
        self.a1.reindexObject()
        transaction.commit()

        data = {'principalid': john.getId()}
        browser.login().visit(self.portal,
                              data=data,
                              view='build_principal_role_tree')
        self.assertEquals('Can add', browser.css('.UserRoles')[0].text)
        self.assertEquals('Can view', browser.css('.UserRoles')[1].text)

    def test_get_user_roles(self):
        folder = create(Builder('folder'))
        john = create(Builder('user').with_roles('Reader', on=folder))
        folder.reindexObject()
        catalog = getToolByName(self.portal, 'portal_catalog')

        result = catalog(principal_with_local_roles=[john.getId()])
        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')

        view()
        self.assertEquals('Can view', view.get_user_roles(result[0]))

    def test_get_group_roles(self):
        folder = create(Builder('folder'))
        john = create(Builder('user'))
        group = create(Builder('group')
                       .titled(u'Group')
                       .with_members(john))
        folder.manage_setLocalRoles(group.getId(), ['Contributor'])
        folder.reindexObject()

        catalog = getToolByName(self.portal, 'portal_catalog')
        result = catalog(principal_with_local_roles=[group.getId()])

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')

        view()
        self.assertEquals([{'title': 'Group', 'roles': 'Can add'}],
                          view.get_group_roles(result[0]))

    @browsing
    def test_build_tree_shows_nested_local_roles(self, browser):
        self._create_structure()
        john = create(Builder('user')
                      .with_roles('Reader', on=self.b11)
                      .with_roles('Reader', on=self.b))
        self.b11.reindexObject()
        self.b.reindexObject()
        transaction.commit()

        data = {'principalid': john.getId()}
        browser.login().visit(self.portal,
                              data=data,
                              view='build_principal_role_tree')

        self.assertEquals('Can view',
                          browser.css('.level1 .UserRoles').first.text)
        self.assertEquals('Can view',
                          browser.css('.level3 .UserRoles').first.text)

    def test_item_template(self):
        self._create_structure()
        john = create(Builder('user').with_roles('Reader', on=self.b11))
        self.b11.reindexObject()

        self.portal.REQUEST.set('principalid', john.getId())
        view = queryMultiAdapter((self.portal, self.portal.REQUEST),
                                 name='build_principal_role_tree')
        view()
        tree = view.build_tree()['children']

        level1 = tree[0]
        self.assertEquals('folder', level1['normalized_portaltype'])

    def test_dexterity_item(self):
        # prepare a fti
        portal_type = 'testtype'
        fti = DexterityFTI(portal_type)
        fti.klass = 'plone.dexterity.content.Item'
        fti.schema = Interface.__identifier__
        self.portal.portal_types._setObject(portal_type, fti)
        register(fti)

        # prepare an item
        item = Item('testitem')
        item.portal_type = portal_type
        self.portal._setObject(item.id, item)
        item = self.portal[item.id]

        john = create(Builder('user').with_roles('Reader', on=item))

        item.reindexObject()

        catalog = getToolByName(self.portal, 'portal_catalog')
        result = catalog(principal_with_local_roles=[john.getId()])

        self.assertEquals(1,
                          len(result),
                          'Expect one entry')
        self.assertEquals(result[0].getPath(),
                          '/'.join(item.getPhysicalPath()))



from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_INTEGRATION_TESTING, TEST_USER_ID_2
import unittest as unittest
from plone.app.testing import TEST_USER_NAME, TEST_USER_ID, login, logout
from ftw.permissionmanager.browser.sharing import SharingView


class TestRemovePermissions(unittest.TestCase):
    """Sharing is well tested by Plone, so just our customized methods"""

    layer = FTW_PERMISSIONMGR_INTEGRATION_TESTING


    def setUp(self):
        # Set up a new user
        portal = self.layer['portal']

        portal.portal_membership.setLocalRoles(
            obj=portal.folder1,
            member_ids=[TEST_USER_ID],
            member_role="Contributor",
            reindex=True)
        portal.folder1.reindexObjectSecurity()

    def test_has_manager_role(self):
        """TEST_USER_ID has manager role from testsetup"""
        portal = self.layer['portal']
        view = SharingView(portal.folder1, portal.folder1.REQUEST)
        self.assertTrue(view.has_manage_portal())

    def test_has_local_role(self):
        portal = self.layer['portal']
        view = SharingView(portal.folder1, portal.folder1.REQUEST)

        # As TEST_USER_ID
        self.assertTrue(view.has_local_role())
        logout()
        login(portal, TEST_USER_ID_2)
        # As TEST_USER_ID_2
        self.assertFalse(view.has_local_role())
        logout()
        login(portal, TEST_USER_NAME)

    def test_role_settings(self):
        """TEST_USER_ID should be Contributor"""
        portal = self.layer['portal']
        view = SharingView(portal.folder1, portal.folder1.REQUEST)
        for entry in view.role_settings():
            if entry['id'] == TEST_USER_ID:
                break
        self.assertTrue(entry['roles']['Contributor'])




from ftw.permissionmanager import permission_manager_factory as _
from ftw.permissionmanager.utils import update_security_of_objects
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter


class CopyUserPermissionsView(BrowserView):

    def __init__(self, *args, **kwargs):
        super(CopyUserPermissionsView, self).__init__(*args, **kwargs)

        self.source_user = None
        self.target_user = None
        self.confirm = None

    def __call__(self, *args, **kwargs):
        self.request.set('disable_border', True)
        form = self.request.form
        self.source_user = form.get('source_user', None)
        self.target_user = form.get('target_user', None)
        self.confirm = form.get('confirm', False)
        if self.source_user and self.target_user and self.confirm:
            return self.copy_permissions()
        return super(CopyUserPermissionsView, self).__call__(*args, **kwargs)

    def search_source_user(self):
        search_term = self.request.form.get('search_source_user', False)
        return self.search_results(search_term)

    def search_target_user(self):
        search_term = self.request.form.get('search_target_user', False)
        return self.search_results(search_term)

    def search_results(self, search_term):
        if not search_term:
            return []
        results = []
        userids = []
        groupids = []
        hunter = getMultiAdapter((self.context, self.request),
                                  name='pas_search')
        # users
        users = hunter.searchUsers(fullname=search_term) + \
            hunter.searchUsers(id=search_term)
        for userinfo in users:
            userid = userinfo['userid']
            user = self.context.acl_users.getUserById(userid)
            if userid not in userids:
                results.append(dict(id=userid,
                                    title=user.getProperty(
                                    'fullname') or user.getId() or userid,
                                    type='user'))
                userids.append(userid)
        # groups
        for groupinfo in hunter.searchGroups(id=search_term):
            groupid = groupinfo['groupid']
            group = self.context.portal_groups.getGroupById(groupid)
            if groupid not in groupids:
                results.append(dict(id=groupid,
                                    title=group.getGroupTitleOrName(),
                                    type='group'))
                groupids.append(groupid)
        return results

    def source_user_title(self):
        return self.getUserOrGroupTitle(self.source_user)

    def target_user_title(self):
        return self.getUserOrGroupTitle(self.target_user)

    def getUserOrGroupTitle(self, user):
        if not user:
            return None
        user = self.context.acl_users.getUserById(user)
        if user:
            return user.getProperty('fullname')
        group = self.context.portal_groups.getGroupById(user)
        if group:
            return group.getGroupTitleOrName()
        return ''

    def copy_permissions(self):
        brains = self.context.portal_catalog(
            path='/'.join(self.context.getPhysicalPath()))

        changed_objects = []
        for brain in brains:
            if self.source_user not in dict(brain.get_local_roles):
                continue

            obj = brain.getObject()
            local_roles = dict(obj.get_local_roles())
            existing_roles = local_roles.get(self.target_user, ())
            new_roles = tuple(set(existing_roles + local_roles.get(self.source_user, ())))
            if new_roles != existing_roles:
                changed_objects.append(obj)
                obj.manage_setLocalRoles(self.target_user, new_roles)
                obj.reindexObject(idxs=['getId'])

        IStatusMessage(self.request).addStatusMessage(
            _(u'Die Berechtigungen wurden kopiert'), type='info')

        update_security_of_objects(changed_objects)
        return self.request.RESPONSE.redirect('@@copy_user_permissions')



from zope.component import getMultiAdapter
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.workflow.browser.sharing import SharingView
from Products.statusmessages.interfaces import IStatusMessage
from ftw.permissionmanager import permission_manager_factory as _
from ftw.permissionmanager.utils import update_security_of_objects


class RemoveUserPermissionsView(SharingView):

    template = ViewPageTemplateFile('templates/remove_user_permissions.pt')

    def __init__(self, *args, **kwargs):
        super(RemoveUserPermissionsView, self).__init__(*args, **kwargs)

        self.search = None
        self.user_selected = None
        self.user = None
        self.confirmed = None

    def __call__(self, *args, **kwargs):
        self.request.set('disable_border', True)
        form = self.request.form
        self.search = form.get('search_user', False)
        self.user_selected = 'user' in form
        self.user = form.get('user', False)
        self.confirmed = 'confirmed' in form
        if self.confirmed:
            self.removePermissions()
            IStatusMessage(self.request).addStatusMessage(
                _(
                    u'Die Berechtigungen wurden erfolgreich entfernt.'),
                    type='info')
            return self.request.RESPONSE.redirect('@@permission_manager')
        return self.template()

    def search_results(self):
        search_term = self.request.form.get('search_term', None)
        if not search_term:
            return []
        results = []
        userids = []
        hunter = getMultiAdapter((self.context, self.request),
                                 name='pas_search')
        # users
        users = hunter.searchUsers(fullname=search_term) + \
            hunter.searchUsers(id=search_term)
        for userinfo in users:
            userid = userinfo['userid']
            user = self.context.acl_users.getUserById(userid)
            if userid not in userids:
                results.append(dict(id = userid,
                                 title = user.getProperty(
                                     'fullname') or user.getId() or userid,
                                 type = 'user'))
                userids.append(userid)
        # groups
        for groupinfo in hunter.searchGroups(id=search_term):
            groupid = groupinfo['groupid']
            group = self.context.portal_groups.getGroupById(groupid)
            results.append(dict(id = groupid,
                             title = group.getGroupTitleOrName(),
                             type = 'group'))
        return results

    def getUserOrGroupTitle(self):
        if not self.user:
            return None
        user = self.context.acl_users.getUserById(self.user)
        if user:
            return user.getProperty('fullname')
        group = self.context.portal_groups.getGroupById(self.user)
        if group:
            return group.getGroupTitleOrName()
        return ''

    def removePermissions(self):
        brains = self.context.portal_catalog(
            path='/'.join(self.context.getPhysicalPath()))
        changed_objects = []
        for brain in brains:
            if self.user in dict(brain.get_local_roles):
                obj = brain.getObject()
                obj.manage_delLocalRoles((self.user, ))
                obj.reindexObject(idxs=['getId'])
                changed_objects.append(obj)

        update_security_of_objects(changed_objects)



from ftw.upgrade import UpgradeStep


class ReindexPrincipalWithLocalRolesIndexForDexteritySupport(UpgradeStep):
    """Reindex principal with local roles index for dexterity support.
    """

    def __call__(self):
        self.install_upgrade_profile()
        self.catalog_rebuild_index('principal_with_local_roles')



from Products.Five import BrowserView


class PermissionManager(BrowserView):

    def __call__(self, *args, **kwargs):
        self.request.set('disable_border', True)
        return super(PermissionManager, self).__call__(*args, **kwargs)



from Acquisition import aq_chain


def update_security_of_objects(objects):
    objects = set(objects)
    for obj in objects.copy():
        parents = set(aq_chain(obj)[1:])
        if parents & objects:
            objects.remove(obj)

    for obj in objects:
        obj.reindexObjectSecurity()



from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_FUNCTIONAL_TESTING
import unittest as unittest
from ftw.testbrowser import browsing
from ftw.builder import create
from ftw.builder import Builder
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
import transaction


class TestRemovePermissions(unittest.TestCase):

    layer = FTW_PERMISSIONMGR_FUNCTIONAL_TESTING

    def setUp(self):
        self.user = create(Builder('user').named("John", "Smith"))
        self.folder = create(Builder('folder'))

    @browsing
    def test_fullname(self, browser):
        browser.login().visit(self.folder, view="sharing")
        browser.fill({"search_term": "Smith J"}).submit()
        self.assertEquals("Smith John",
                          browser.css("#user-group-sharing-settings a").text[0])

    @browsing
    def test_mail(self, browser):
        browser.login().visit(self.folder, view="sharing")
        browser.fill({"search_term": "john@smith"}).submit()
        self.assertEquals("Smith John",
                          browser.css("#user-group-sharing-settings a").text[0])

    @browsing
    def test_fullname_removed(self, browser):
        registry = getUtility(IRegistry)
        fields = (u'email',)
        registry["ftw.permissionmanager.fields_to_search"] = fields
        transaction.commit()
        browser.login().visit(self.folder, view="sharing")
        browser.fill({"search_term": "Smith J"}).submit()
        self.assertEquals(0,
                          len(browser.css("#user-group-sharing-settings a").text))

        browser.fill({"search_term": "john@smith"}).submit()
        self.assertEquals("Smith John",
                          browser.css("#user-group-sharing-settings a").text[0])



from ftw.upgrade import UpgradeStep


class InstallNewIndex(UpgradeStep):

    def __call__(self):
        catalog = self.getToolByName('portal_catalog')
        catalog.addIndex('principal_with_local_roles', 'KeywordIndex')
        self.catalog_reindex_objects({'is_folderish': True},
                                     idxs=['principal_with_local_roles'])



from ftw.upgrade import UpgradeStep
from ftw.upgrade.progresslogger import ProgressLogger
from ftw.upgrade.step import LOG


class InstallMetadata(UpgradeStep):

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.permissionmanager.upgrades:2302')
        self.update_workflow_id_metadata()

    def update_workflow_id_metadata(self):
        brains = self.catalog_unrestricted_search({}, full_objects=False)
        for brain in ProgressLogger('Reindex workflow_id metadata', brains):
            try:
                obj = self.catalog_unrestricted_get_object(brain)
            except AttributeError:
                LOG.warning('Could not get {0}'.format(brain.getPath()))
                continue

            catalog = self.getToolByName('portal_catalog')
            catalog.reindexObject(obj, idxs=['id'], update_metadata=True)



from Products.CMFCore.utils import getToolByName
from ftw.builder import Builder
from ftw.builder import create
from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_INTEGRATION_TESTING
from unittest import TestCase


class TestCatalog(TestCase):
    layer = FTW_PERMISSIONMGR_INTEGRATION_TESTING

    def get_metadata_for(self, obj):
        catalog = getToolByName(obj, 'portal_catalog')
        rid = catalog.getrid('/'.join(obj.getPhysicalPath()))
        return catalog.getMetadataForRID(rid)

    def test_workflow_id_metadata(self):
        wftool = getToolByName(self.layer['portal'], 'portal_workflow')
        wftool.setChainForPortalTypes(['Document'], 'plone_workflow')

        obj = create(Builder('document'))
        self.assertDictContainsSubset(
            {'workflow_id': 'plone_workflow'},
            self.get_metadata_for(obj))

    def test_workflow_id_metadata_is_None_when_no_workflow_configured(self):
        obj = create(Builder('document'))
        self.assertDictContainsSubset(
            {'workflow_id': None},
            self.get_metadata_for(obj))



from ftw.builder import Builder
from ftw.builder import create
from ftw.permissionmanager.testing import FTW_PERMISSIONMGR_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from unittest import TestCase


class TestReindexObjectSecurityPatch(TestCase):

    layer = FTW_PERMISSIONMGR_INTEGRATION_TESTING

    def setUp(self):
        self.catalog = getToolByName(self.layer['portal'], 'portal_catalog')

    def _get_brain(self, obj):
        return self.catalog(path='/'.join(obj.getPhysicalPath()))[0]

    def test_if_metadata_are_up_to_date(self):
        folder = create(Builder('folder'))
        john = create(Builder('user').with_roles('Reader', on=folder))
        folder.reindexObjectSecurity()
        folder.reindexObject(idxs=['getId'])

        brain = self._get_brain(folder)

        self.assertIn((john.getId(), ('Reader', )), brain.get_local_roles)
        self.assertTrue(brain.isLocalRoleAcquired)

    def test_index_is_up_to_date(self):
        folder = create(Builder('folder'))
        john = create(Builder('user').with_roles('Reader', on=folder))
        folder.reindexObjectSecurity()

        self.assertEquals(
            1,
            len(self.catalog(principal_with_local_roles=john.getId())),
            'Expect one catalog entry')



from ftw.permissionmanager import permission_manager_factory as _
from plone.app.workflow.interfaces import ISharingPageRole
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getUtilitiesFor
import csv
import StringIO

DEFAULT_ROLES = ['Owner', ]


class ImportExportPermissionsView(BrowserView):

    READONLY = ('Name', 'Title', )


    def __init__(self, *args, **kwargs):
        super(ImportExportPermissionsView, self).__init__(*args, **kwargs)
        self.recursive = None
        self.relative_paths = None
        self.structure_only = None
        self.portal_membership = getToolByName(self.context, 'portal_membership')

    def __call__(self, *args, **kwargs):
        self.request.set('disable_border', True)
        # Export
        self.recursive = self.request.get('recursive') and True
        self.relative_paths = self.request.get('relative_paths') and True
        self.structure_only = self.request.get('structure_only') and True
        self.encoding = self.request.get('select_encoding')
        if self.request.get('export'):
            return self.export()
        elif self.request.get('import'):
            self.startImport()
        else:
            self.recursive = True
            self.relative_paths = False
            self.structure_only = False
        return super(ImportExportPermissionsView, self).__call__(
            *args, **kwargs)

    @memoize
    def get_roles(self):
        roles = []
        for name, _utility in getUtilitiesFor(ISharingPageRole):
            roles.append(name)
        return roles + DEFAULT_ROLES

    @memoize
    def get_fieldnames(self):
        return ['Name', 'Userid', 'Title'] + self.get_roles() + ['Path']

    def export(self):
        query = {}
        path = {
            'query': '/'.join(self.context.getPhysicalPath()),
        }
        if not self.recursive:
            path['depth'] = 0
        query['path'] = path
        if self.structure_only:
            query['is_folderish'] = True
        objects = [b.getObject() for b in self.context.portal_catalog(**query)]
        file_ = StringIO.StringIO()
        fieldnames = self.get_fieldnames()
        writer = csv.DictWriter(file_,
                                fieldnames=fieldnames,
                                dialect='excel_ger')

        labels = []
        for name in fieldnames:
            if name in ImportExportPermissionsView.READONLY:
                name += ' (RO)'
            labels.append(name)

        writer.writerow(dict(zip(fieldnames, labels)))
        for obj in objects:
            self.export_object(writer, obj)
        file_.seek(0)
        data = file_.read()
        #
        self.request.RESPONSE.setHeader(
            'Content-Type', 'text/csv; charset=%s' % self.encoding)
        filename = '%s-permissions.csv' % self.context.id
        self.request.RESPONSE.setHeader(
            'Content-disposition', 'attachment; filename=%s' % filename)
        return data

    def export_object(self, writer, obj):
        path = '/'.join(obj.getPhysicalPath())
        if self.relative_paths:
            contextPath = '/'.join(self.context.getPhysicalPath())
            path = '...%s' % path[len(contextPath):]
        for user, roles in obj.get_local_roles():
            member = self.portal_membership.getMemberById(user)
            fullname = member and member.getProperty('fullname', user) or user
            row = {
                'Name': fullname.decode('utf-8').encode(self.encoding),
                'Userid': user.encode(self.encoding),
                'Title': obj.Title().decode('utf-8').encode(self.encoding),
                'Path': path,
            }
            for role in roles:
                if role in self.get_roles():
                    row[role] = 'X'
            writer.writerow(row)

    def startImport(self):
        _file = self.request.get('file')
        data = _file.read().replace('\r\n', '\n').replace('\r', '\n')
        data = StringIO.StringIO(data)
        dialect = csv.Sniffer().sniff(data.readline())
        data.seek(0)
        reader = csv.DictReader(data, fieldnames=self.get_fieldnames(),
                            dialect=dialect)
        titles = None
        rows_imported = 0
        for row in reader:
            if not titles:
                titles = row
                continue
            if self.setPermissions(row):
                rows_imported += 1
        IStatusMessage(self.request).addStatusMessage(
            _(
                u'Es wurden ${rows_imported} Berechtigungen gesetzt.',
                mapping=dict(rows_imported=rows_imported)),
            type='info')

        self.context.reindexObjectSecurity()

    def setPermissions(self, row):
        try:
            obj = self.getObjectByPath(row)
        except (AttributeError, KeyError):
            IStatusMessage(self.request).addStatusMessage(
                _(
                    u'Objekt konnte nicht gefunden werden: ${path}',
                    mapping=dict(path=row['Path'])),
                type='error')
            return

        user = row['Userid'].decode(self.encoding)
        roles = []
        for role in self.get_roles():
            if len(row[role])>0:
                roles.append(role)
        if len(roles)>0:
            obj.manage_setLocalRoles(user, roles)
            obj.reindexObject(idxs=['getId'])
        else:
            obj.manage_delLocalRoles((user, ))
            obj.reindexObject(idxs=['getId'])
        return True

    def getObjectByPath(self, row):
        path = row['Path'].decode(self.encoding)
        path = path.encode('utf-8')
        if path.startswith('...'):
            # relative paths
            path = path[len('.../'):]
        obj = self.context.restrictedTraverse(path)
        return obj



from zope.component import getMultiAdapter
from ftw.permissionmanager.testing import (
    FTW_PERMISSIONMGR_INTEGRATION_TESTING,
    TEST_USER_ID_2)
from plone.app.testing import TEST_USER_ID
import unittest as unittest
from plone.app.workflow.interfaces import ISharingPageRole
from zope.component import getUtilitiesFor
import StringIO


class TestCopyPermissions(unittest.TestCase):

    layer = FTW_PERMISSIONMGR_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        # Create document with TEST_USER_ID
        portal.folder1.folder2.invokeFactory(
            'Document', 'document1', title='Document 1')

        # Give TEST_USER_ID_2 some roles
        portal.portal_membership.setLocalRoles(
            obj=portal.folder1.folder2,
            member_ids=[TEST_USER_ID_2],
            member_role="Reader",
            reindex=True)
        member2 = portal.portal_membership.getMemberById(TEST_USER_ID_2)
        member2.setMemberProperties({'fullname': 'Test \xc3\x9cser 2'})

        portal.portal_membership.setLocalRoles(
            obj=portal.folder1.folder2.document1,
            member_ids=[TEST_USER_ID_2],
            member_role="Editor",
            reindex=True)

    def test_copy_permission_view(self):
        portal = self.layer['portal']
        view = getMultiAdapter(
            (portal.folder1, portal.folder1.REQUEST),
            name="import_export_permissions")
        self.assertTrue(view.__name__ == 'import_export_permissions')

    def test_get_roles(self):
        portal = self.layer['portal']
        view = getMultiAdapter(
            (portal.folder1, portal.folder1.REQUEST),
            name="import_export_permissions")

        roles = view.get_roles()
        for name, utility in getUtilitiesFor(ISharingPageRole):
            self.assertTrue(name in roles)

    def test_export_structure(self):
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('recursive', '1')
        request.set('relative_paths', '')
        request.set('structure_only', '')
        request.set('select_encoding', 'utf-8')

        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        roles = view.get_roles()
        # first row contains headers
        head = data.split('\r\n')[0].split(';')
        self.assertEqual(head[0], 'Name (RO)')
        self.assertEqual(head[1], 'Userid')
        self.assertEqual(head[2], 'Title (RO)')
        self.assertEqual(head[-1], 'Path')
        for role in roles:
            self.assertTrue(role in head)

        # All records should have the same len as the head
        for record in data.split('\r\n')[1:-1]:
            self.assertTrue(len(head) == len(record.split(';')))


    def test_export_data(self):
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('recursive', '1')
        request.set('relative_paths', '')
        request.set('structure_only', '')
        request.set('select_encoding', 'utf-8')

        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        head = data.split('\r\n')[0].split(';')

        # TEST_USER_ID ist Owner of folder1, folder2 and document1
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            if record[1] == TEST_USER_ID:
                index = head.index('Owner')
                self.assertEqual(
                    record[index], 'X')

        # TEST_USER_ID_2 has Reader role folder2
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            if record[1] == TEST_USER_ID:
                continue
            if record[1] == TEST_USER_ID_2 and record[-1] == '/plone/folder1/folder2':
                index = head.index('Reader')
                self.assertEqual(
                    record[index], 'X')
                self.assertEqual(['Test \xc3\x9cser 2', TEST_USER_ID_2],
                                 record[:2])
            elif record[1] == TEST_USER_ID_2 and record[-1] == '/plone/folder1/folder2/document1':
                index = head.index('Editor')
                self.assertEqual(
                    record[index], 'X')

    def test_export_structure_only(self):
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('export', '1')
        request.set('recursive', '1')
        request.set('relative_paths', '')
        request.set('structure_only', '1')
        request.set('select_encoding', 'utf-8')

        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            self.assertFalse('document1' in record[-1]) # Path


    def test_export_relative_path(self):
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('export', '1')
        request.set('recursive', '1')
        request.set('relative_paths', '1')
        request.set('structure_only', '')
        request.set('select_encoding', 'utf-8')

        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            self.assertTrue(record[-1].startswith('...')) # Path

    def test_export_not_recursiv(self):
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('export', '1')
        request.set('recursive', '')
        request.set('relative_paths', '')
        request.set('structure_only', '')
        request.set('select_encoding', 'utf-8')
        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            self.assertTrue(record[-1] == '/plone/folder1') # Path

    def test_import_permissions(self):
        # First do a export
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('select_encoding', 'utf-8')
        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        head = data.split('\r\n')[0].split(';')
        # Change permission - set TEST_USER_ID_2 as Owner of folder2 and
        # docuement1
        new_records = []
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            if record[1] == TEST_USER_ID_2:
                index = head.index('Owner')
                new_record = record
                new_record.pop(index)
                new_record.insert(index, 'X')
                new_records.append(';'.join(new_record))
        data += '\r\n'.join(new_records)

        # Prepare import
        request.set('import', '1')
        request.set('file', StringIO.StringIO(data))
        request.set('select_encoding', 'utf-8')
        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        self.assertIn(
            'Owner',
            portal.folder1.folder2.get_local_roles_for_userid(TEST_USER_ID_2))
        self.assertIn(
            'Owner',
            portal.folder1.folder2.document1.get_local_roles_for_userid(TEST_USER_ID_2))

    def test_import_permissions_not_existing_path(self):
        # First do a export
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('select_encoding', 'utf-8')
        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        data = view.export()
        data += 'test_user_1_;test_user_1_;Document 1;;;;;X;/plone/folder111/folder222/document111\r\n'
        head = data.split('\r\n')[0].split(';')
        # Change permission - set TEST_USER_ID_2 as Owner of folder2 and
        # docuement1
        new_records = []
        for line in data.split('\r\n')[1:-1]:
            record = line.split(';')
            if record[1] == TEST_USER_ID_2:
                index = head.index('Owner')
                new_record = record
                new_record.pop(index)
                new_record.insert(index, 'X')
                new_records.append(';'.join(new_record))
        data += '\r\n'.join(new_records)

        # Prepare import
        request.set('import', '1')
        request.set('file', StringIO.StringIO(data))
        request.set('select_encoding', 'utf-8')
        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        self.assertIn(
            'Owner',
            portal.folder1.folder2.get_local_roles_for_userid(TEST_USER_ID_2))
        self.assertIn(
            'Owner',
            portal.folder1.folder2.document1.get_local_roles_for_userid(TEST_USER_ID_2))

    def test_export_changeencoding(self):
        portal = self.layer['portal']
        # set up request
        request = portal.folder1.REQUEST
        request.set('export', '1')
        request.set('recursive', '1')
        request.set('relative_paths', '1')
        request.set('structure_only', '')
        request.set('select_encoding', 'latin1')

        view = getMultiAdapter(
            (portal.folder1, request),
            name="import_export_permissions")
        view()
        view.export()
        self.assertEqual(view.request.RESPONSE.getHeader('Content-Type'), 'text/csv; charset=latin1')
        request.set('select_encoding', 'MacRoman')
        view()
        view.export()
        self.assertEqual(view.request.RESPONSE.getHeader('Content-Type'), 'text/csv; charset=MacRoman')



from OFS.interfaces import IItem
from plone.indexer import indexer
from Products.CMFCore.utils import getToolByName


@indexer(IItem)
def global_get_local_roles(object):
    return object.get_local_roles()


@indexer(IItem)
def principal_with_local_roles(obj):
    return [user_roles[0] for user_roles in obj.get_local_roles()]


@indexer(IItem)
def isLocalRoleAcquired(object):
    if getattr(object, '__ac_local_roles_block__', None):
        return False
    return True


@indexer(IItem)
def workflow_id(obj):
    wftool = getToolByName(obj, 'portal_workflow')
    workflows = wftool.getWorkflowsFor(obj)

    if len(workflows) == 0:
        return None

    return workflows[0].id



from setuptools import setup, find_packages
import os

version = '3.0.2'
maintainer = 'Mathias Leimgruber'

tests_require = [
    'plone.app.contenttypes',
    'plone.app.testing',
    'Products.DateRecurringIndex',
    'ftw.testbrowser',
    'ftw.testing',
    'ftw.builder']

setup(name='ftw.permissionmanager',
      version=version,
      description="",
      long_description=open("README.rst").read() + "\n" + \
          open(os.path.join("docs", "HISTORY.txt")).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.permissionmanager',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'Plone',
        'setuptools',
        'Products.CMFPlone >= 4.3b',
        'ftw.upgrade',
        'ftw.profilehook',
        # -*- Extra requirements: -*-
        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from ftw.builder.content import register_dx_content_builders
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from plone.testing import z2
from zope.configuration import xmlconfig


TEST_USER_ID_2 = '_test_user_2_'
TEST_USER_PW_2 = 'secret'
TEST_GROUP_ID = 'test_group'
TEST_GROUP_ID_2 = 'test_group_2'


class FtwPermissionmanagerLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.permissionmanager')
        z2.installProduct(app, 'Products.DateRecurringIndex')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.permissionmanager:default')
        applyProfile(portal, 'plone.app.contenttypes:default')
        register_dx_content_builders(force=True)

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        # Create one folder with one item
        portal.invokeFactory('Folder', 'folder1', title='Folder1')
        portal.folder1.invokeFactory('Folder', 'folder2', title='Folder2')

        # Add a new user
        portal.portal_registration.addMember(TEST_USER_ID_2, TEST_USER_PW_2)


FTW_PERMISSIONMANAGER_FIXTURE = FtwPermissionmanagerLayer()
FTW_PERMISSIONMGR_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_PERMISSIONMANAGER_FIXTURE, ),
    name="ftw.permissionmanager:Integration")


FTW_PERMISSIONMGR_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_PERMISSIONMANAGER_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.permissionmanager:Functional")



from Products.CMFCore import CMFCatalogAware
from zope.i18nmessageid import MessageFactory
import csv


permission_manager_factory = MessageFactory('ftw.permissionmanager')


class excel_ger(csv.excel):
    """Extend csv library with custem dialect excel_ger"""
    delimiter = ';'
csv.register_dialect('excel_ger', excel_ger)


def initialize(context):
    """Initializer called when used as a Zope 2 product."""
    register_local_roles_index()


def register_local_roles_index():
    name = 'principal_with_local_roles'
    klass = CMFCatalogAware.CMFCatalogAware

    if name in klass._cmf_security_indexes:
        return

    indexes = list(klass._cmf_security_indexes)
    indexes.append(name)
    klass._cmf_security_indexes = tuple(indexes)



from Products.CMFCore.utils import getToolByName
import os


class Treeify(object):
    """ Generates a tree structure out of a set of catalog brains incl. all
        parents.

        Example: Give is the <mybrain instance4>

        >>> {'children': [
        ...    {'group_roles': [],
        ...     'item': <mybrain instance1>,
        ...     'user_roles': '',
        ...     'children': [
        ...         {'group_roles': [],
        ...          'item': <mybrain instance2>,
        ...          'user_roles': '',
        ...          'children': [
        ...              {'group_roles': [],
        ...               'item': <mybrain instance3>,
        ...               'user_roles': '',
        ...               'children': [
        ...                   {'group_roles': [],
        ...                    'item': <mybrain instance4>,
        ...                    'user_roles': u'Can view',
        ...                    'children': []}]}]}]}]}
    """

    def __init__(self, brains, root_path, callback):
        self._brains = brains
        self._root_path = root_path
        self._callback = callback
        self._brains_by_path = {}
        self._nodes_by_path = {}
        self.context = None

    def __call__(self, context):
        self.context = context
        self._load_brain_cache()
        for brain in self._brains:
            self._update_node(brain.getPath())
        return self._nodes_by_path[self._root_path]

    def _load_brain_cache(self):
        for brain in self._brains:
            self._brains_by_path[brain.getPath()] = brain

    def _get_node(self, path):
        if path not in self._nodes_by_path:
            self._update_node(path)
        return self._nodes_by_path[path]

    def _update_node(self, path):
        if path in self._nodes_by_path:
            return

        node = {'children': []}

        site_root_path = '/'.join(
            getToolByName(self.context,
                          'portal_url').getPortalObject().getPhysicalPath())
        if path == site_root_path:
            self._nodes_by_path[path] = node
        else:
            brain = self._get_brain(path)
            node = {'children': []}
            self._callback(brain, node)
            self._nodes_by_path[path] = node

        if path != self._root_path:
            parent_path = os.path.dirname(path)
            self._get_node(parent_path)['children'].append(node)

    def _get_brain(self, path):
        if path not in self._brains_by_path:
            catalog = getToolByName(self.context, 'portal_catalog')
            self._brains_by_path[path] = catalog.unrestrictedSearchResults(
                {'path': {'query': path, 'depth': 0}})[0]
        return self._brains_by_path[path]



from Acquisition import aq_inner
from plone.app.workflow.browser.sharing import SharingView as base
from plone.app.workflow.interfaces import ISharingPageRole
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode, getSiteEncoding
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtilitiesFor
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from plone.app.workflow.browser.sharing import merge_search_results
from itertools import chain


class SharingView(base):
    """This sharing view works a bit diffrent than the one from plone
       The search result is displayed in his own table.
       If a role is selected in search result table, the entry will be added
       to current_settings, this (kind of) saves the result for the user
    """

    # index: New integration for Plone >= 4.3.2
    # template: Backward compatibility Plone < 4.3.2
    index = template = ViewPageTemplateFile('templates/sharing.pt')

    def __call__(self, *args, **kwargs):
        self.disable_right_column()
        return super(SharingView, self).__call__(*args, **kwargs)

    def disable_right_column(self):
        self.request.set('disable_plone.rightcolumn', 1)

    def handle_form(self):
        if self.request.form.get('form.button.Save', None):
            # Clear the search form and hide the search results when the
            # sharing form is submitted.
            self.request.form.pop('search_term', None)
        return super(SharingView, self).handle_form()

    def has_manage_portal(self):
        return self.context.portal_membership.checkPermission(
            'ManagePortal',
            self.context)

    def has_local_role(self, role=''):
        local_roles = self.context.aq_explicit.get_local_roles()
        authenticated_member = \
            self.context.portal_membership.getAuthenticatedMember().id
        for userid, roles in local_roles:
            if (authenticated_member == userid and role in roles) \
                or self.has_manage_portal():
                return True
        return False


    def user_search_results(self):
        """Return search results for a query to add new users.

        Returns a list of dicts, as per role_settings().
        """

        def search_for_principal(hunter, search_term):
            registry = getUtility(IRegistry)
            fields = registry["ftw.permissionmanager.fields_to_search"]
            return merge_search_results(chain(*[hunter.searchUsers(**{field: search_term})
                for field in fields]), 'userid')

        def get_principal_by_id(user_id):
            acl_users = getToolByName(self.context, 'acl_users')
            return acl_users.getUserById(user_id)

        def get_principal_title(user, default_title):
            return user.getProperty('fullname') or user.getId() or default_title

        return self._principal_search_results(search_for_principal,
            get_principal_by_id, get_principal_title, 'user', 'userid')


    def role_settings(self):
        """Only return current settings"""
        results = self.existing_role_settings()
        encoding = getSiteEncoding(aq_inner(self.context))
        static = {}
        index = None
        # Puts AuthenticatedUsers group to position 0
        for item in results:
            if item['id'] == 'AuthenticatedUsers':
                index = results.index(item)
                break
        if index != -1 and index is not None:
            static = results.pop(index)
        results.sort(
            lambda x, y: cmp(
                safe_unicode(x["title"], encoding).lower(),
                safe_unicode(y["title"], encoding).lower()))
        # Only show AuthenticatedUsers group users with ManagePortal
        # perission
        # XXX: This should be configurable
        if static and self.has_manage_portal():
            results.insert(0, static)
        return results

    def search_result(self):
        """Returns only the search result for the new search result table"""

        # XXX: This method is inefficient. To many interations and
        # manipulations. # If someone really understands what to do,
        # please do it.

        mtool = getToolByName(self.context, 'portal_membership')
        gtool = getToolByName(self.context, 'portal_groups')
        user_results = self.user_search_results()
        group_results = self.group_search_results()
        current_settings = user_results + group_results

        requested = self.request.form.get('entries', [])

        # Remove real/saved roles from requested
        for item in self.existing_role_settings():
            index = None
            for r_item in requested:
                if item['id'] == r_item['id']:
                    index = requested.index(r_item)
                    break
            if index is not None:
                requested.pop(index)

        if requested is not None:
            knownroles = [r['id'] for r in self.roles()]
            settings = {}
            result_like_settings = {}
            for entry in requested:
                roles = []
                roles_for_settings = {}
                for r in knownroles:
                    if entry.get('role_%s' % r, False):
                        roles.append(r)
                        roles_for_settings[r] = True
                    else:
                        roles_for_settings[r] = False
                    settings[(entry['id'], entry['type'])] = roles
                # Add requested entries to current_settings if there is one or
                #  more roles selected
                # This is kind a temporary storage for search results.
                # It allows you to do multible search queries and you will not
                # lose allready selected roles for user/groups
                if roles and not self.request.get('form.button.Save', None):
                    # get group title or user fullname
                    if entry['type'] == 'user':
                        member = mtool.getMemberById(entry['id'])
                        title = '%s' % (
                            member.getProperty('fullname', entry['id']))
                    else:
                        group = gtool.getGroupById(entry['id'])
                        title = group.getGroupTitleOrName()
                    result_like_settings = {
                        'id': entry['id'],
                        'type': entry['type'],
                        'title': title,
                        'roles': roles_for_settings}

                    # Add or update settings
                    updated = False
                    for given_entry in current_settings:
                        if result_like_settings['id'] == given_entry['id']:
                            given_entry.update(result_like_settings)
                            updated = True
                            break
                    if not updated:
                        current_settings.append(result_like_settings)
            for entry in current_settings:
                desired_roles = settings.get(
                    (entry['id'], entry['type']),
                    None)

                if desired_roles is None:
                    continue
                for role in entry["roles"]:
                    if entry["roles"][role] in [True, False]:
                        entry["roles"][role] = role in desired_roles

        encoding = getSiteEncoding(aq_inner(self.context))
        current_settings.sort(
            lambda x, y: cmp(
                safe_unicode(x["title"], encoding).lower(),
                safe_unicode(y["title"], encoding).lower()))

        return current_settings
